<?php

namespace Litecms\Information\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Information\Http\Requests\InformationRequest;
use Litecms\Information\Interfaces\InformationRepositoryInterface;
use Litecms\Information\Models\Information;

/**
 * Resource controller class for information.
 */
class InformationResourceController extends BaseController
{

    /**
     * Initialize information resource controller.
     *
     * @param type InformationRepositoryInterface $information
     *
     * @return null
     */
    public function __construct(InformationRepositoryInterface $information)
    {
        parent::__construct();
        $this->repository = $information;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Information\Repositories\Criteria\InformationResourceCriteria::class);
    }

    /**
     * Display a list of information.
     *
     * @return Response
     */
    public function index(InformationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Information\Repositories\Presenter\InformationPresenter::class)
                ->$function();
        }

        $information = $this->repository->paginate();

        return $this->response->title(trans('information::information.names'))
            ->view('information::information.index', true)
            ->data(compact('information', 'view'))
            ->output();
    }

    /**
     * Display information.
     *
     * @param Request $request
     * @param Model   $information
     *
     * @return Response
     */
    public function show(InformationRequest $request, Information $information)
    {

        if ($information->exists) {
            $view = 'information::information.show';
        } else {
            $view = 'information::information.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('information::information.name'))
            ->data(compact('information'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new information.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InformationRequest $request)
    {

        $information = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('information::information.name')) 
            ->view('information::information.create', true) 
            ->data(compact('information'))
            ->output();
    }

    /**
     * Create new information.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InformationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $information                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('information::information.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('information/information/' . $information->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/information/information'))
                ->redirect();
        }

    }

    /**
     * Show information for editing.
     *
     * @param Request $request
     * @param Model   $information
     *
     * @return Response
     */
    public function edit(InformationRequest $request, Information $information)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('information::information.name'))
            ->view('information::information.edit', true)
            ->data(compact('information'))
            ->output();
    }

    /**
     * Update the information.
     *
     * @param Request $request
     * @param Model   $information
     *
     * @return Response
     */
    public function update(InformationRequest $request, Information $information)
    {
        try {
            $attributes = $request->all();

            $information->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('information::information.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('information/information/' . $information->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('information/information/' . $information->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the information.
     *
     * @param Model   $information
     *
     * @return Response
     */
    public function destroy(InformationRequest $request, Information $information)
    {
        try {

            $information->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('information::information.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('information/information/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('information/information/' . $information->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple information.
     *
     * @param Model   $information
     *
     * @return Response
     */
    public function delete(InformationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('information::information.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('information/information'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/information/information'))
                ->redirect();
        }

    }

    /**
     * Restore deleted information.
     *
     * @param Model   $information
     *
     * @return Response
     */
    public function restore(InformationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('information::information.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/information/information'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/information/information/'))
                ->redirect();
        }

    }

}
